import tensorflow as tf

from src.enum.activierungsfunktionEnum import Aktivierungsfunktion
from src.enum.optimierungsverfahren import Optimierungsverfahren


# define cnn model
def define_model(config_knn):
    # Vortrainierte VGG16-Modell wird geladen
    model = tf.keras.applications.VGG16(include_top=False, input_shape=(224, 224, 3))
    # Die geladenen Schichten müssen nicht neu trainiert werden
    for layer in model.layers:
        layer.trainable = False
    # Die Fully-Connected-Schichten werden definiert
    flat = tf.keras.layers.Flatten()(model.layers[-1].output)
    output_layer_1 = get_next_layer(128, config_knn.activation_function_128_units, flat)
    output_layer_2 = get_next_layer(1, config_knn.activation_function_1_units, output_layer_1)
    # Die Schichten des Modells werden definiert
    model = tf.keras.models.Model(inputs=model.inputs, outputs=output_layer_2)

    # Das Optimierungsverfahren wird definiert
    opt = get_optimization_method(config_knn.optimization_method)
    # Die Klassifizierungsmetriken und die Loss-Funktion werden definiert
    recall_not_oos = tf.keras.metrics.Recall(name='recall_not_oos', class_id=0)
    recall_oos = tf.keras.metrics.Recall(name='recall_oos', class_id=1)
    model.compile(optimizer=opt, loss='binary_crossentropy', metrics=['accuracy', recall_not_oos, recall_oos])
    return model


def get_next_layer(units, activation_function, flat_or_dense):
    if activation_function == Aktivierungsfunktion.ReLU:
        return tf.keras.layers.Dense(units,
                                     activation=activation_function,
                                     kernel_initializer='he_uniform')(flat_or_dense)
    if activation_function == Aktivierungsfunktion.Sigmoid:
        return tf.keras.layers.Dense(units, activation='sigmoid')(flat_or_dense)
    return None


def get_optimization_method(optimization_method):
    if optimization_method == Optimierungsverfahren.SGD:
        return tf.keras.optimizers.SGD(learning_rate=0.001, momentum=0.9)
    if optimization_method == Optimierungsverfahren.Adam:
        return tf.keras.optimizers.Adam(learning_rate=0.001)
    return None
